// See COPYING for license information.

#include <sys/stat.h>
#include "defs.h"
#include "CVersionCompare.h"
#include "CParser.h"
#include "CMirrorHandler.h"
#include "CConfigFile.h"
#include "CUpgrader.h"

CUpgrader::CUpgrader(const bool& all, std::vector<std::string>& files)
{
	// Print URL fancily.
	std::string	_mirr = CConfigFile::Instance().GetValue("CurrentMirror")[0];
	int	_snap = _mirr.rfind("snapshot");
	int	_curr = _mirr.rfind("current");
	if ( _snap >= 0 )
		std::cout << "Mirror: " << _mirr.substr(0, _snap) << VT_COLOR_DOWNGRADE << "snapshot" << _mirr.substr(_snap + 8) << std::endl;
	else if ( _curr >= 0 )
		std::cout << "Mirror: " << _mirr.substr(0, _curr) << VT_COLOR_UPGRADE << "current" << _mirr.substr(_curr + 7) << std::endl;

	// Are you sure you want to go through with this?
	for (;;) {
		std::cout << VT_COLOR_DOWNGRADE << "Warning!" << VT_COLOR_NORMAL << " You are about to get all of the upgrades";
		if ( all )	std::cout << " and downgrades\n";
		else		std::cout << std::endl;
		std::cout << "that the mirror has to offer.  Do you want to proceed? [y/n]: ";
		std::cout.flush();
		std::string	_choice;
		std::getline(std::cin, _choice);
		if ( !_choice.find('n') )	return;
		if ( !_choice.find('y') )	break;
	}

	reload_old_mirror();

	CVersionCompare	_vcomp;
	CParser		_parser;
	package_map&	_packages = _parser.Packages;

	for ( package_map::const_iterator _iter = _packages.begin(); _iter != _packages.end(); ++_iter ) {
		bool	_upgrade = false;
		bool	_downgrade = false;
		// Find out if this is an upgrade, a downgrade, or neither.
		if ( _iter->second.mirror_version.empty() || _iter->second.installed_version.empty() )
			continue;
		else if ( _vcomp.Compare(_iter->second.mirror_version, _iter->second.installed_version) )
			_upgrade = true;
		else if ( _vcomp.Compare(_iter->second.installed_version, _iter->second.mirror_version) )
			_downgrade = true;

		// Add it to the list of packages to be installed if it
		// fits the filter.
		if ( all && (_upgrade || _downgrade) )	files.push_back(_iter->first);
		else if ( !all && _upgrade ) 		files.push_back(_iter->first);
	}

	std::cout << "Found " << files.size() << " packages to upgrade.\n";
}

void CUpgrader::reload_old_mirror()
{	// Reload mirror if the PACKAGES.TXT file is old.
	struct stat	_dbstats;
	stat(TMPFILE("PACKAGES.TXT").c_str(), &_dbstats);
	if ( time(NULL) - _dbstats.st_mtime > PACKAGES_TXT_MAX_AGE )	CMirrorHandler(false);
}
